@@ -1,14 +1,24 @@
 import logging
 import datetime
 import sqlalchemy as sa
-from formencode import htmlfill
-
-from formbar.fahelpers import get_fieldset, get_data
-from formbar.renderer import FormRenderer, FieldRenderer, get_renderer
+from formencode import htmlfill, variabledecode
+from formbar.renderer import FormRenderer, get_renderer
+from formbar.rules import Rule, Parser
 
 log = logging.getLogger(__name__)
 
 
+def get_attributes(cls):
+    return [prop.key for prop in sa.orm.class_mapper(cls).iterate_properties
+            if isinstance(prop, sa.orm.ColumnProperty)
+            or isinstance(prop, sa.orm.RelationshipProperty)]
+
+
+def get_relations(cls):
+    return [prop.key for prop in sa.orm.class_mapper(cls).iterate_properties
+            if isinstance(prop, sa.orm.RelationshipProperty)]
+
+
 class Error(Exception):
     """Base class for exceptions in this module."""
     pass
@@ -25,7 +35,7 @@ class StateError(Error):
 
 
 class Validator(object):
-    """Docstring for Validator """
+    """Docstring for Validator"""
 
     def __init__(self, field, error, callback):
         """@todo: to be defined
@@ -60,7 +70,7 @@ class Form(object):
     """
 
     def __init__(self, config, item=None, dbsession=None, translate=None,
-                 change_page_callback={}, renderers={}):
+                 change_page_callback={}, renderers={}, request=None):
         """Initialize the form with ``Form`` configuration instance and
         optional an SQLAlchemy mapped object.
 
@@ -75,22 +85,20 @@ class Form(object):
         provided to the form to render specific form elements. The key
         is the type of the renderer as named in the formular
         configuration.
+        :request: Current request (See
+        http://docs.pylonsproject.org/projects/pyramid/en/latest/api/request.html
+        when using in connection with ringo)
 
         """
         self._config = config
         self._item = item
         self._dbsession = dbsession
+        self._request = request
         if translate:
             self._translate = translate
         else:
             self._translate = lambda msgid: msgid
 
-        self.fs = get_fieldset(item, config, dbsession)
-        """FormAlchemy fieldset"""
-        self.data = get_data(self.fs)
-        """After submission this Dictionary will contain either the
-        validated data on successfull validation or the origin submitted
-        data."""
         self.validated = False
         """Flag to indicate if the form has been validated. Init value
         is False.  which means no validation has been done."""
@@ -113,6 +121,61 @@ class Form(object):
         """Dictionary with external provided custom renderers."""
         self.fields = self._build_fields()
         """Dictionary with fields."""
+        self.data = self.serialize(self._get_data_from_item())
+        """After submission this Dictionary will contain either the
+        validated data on successfull validation or the origin submitted
+        data."""
+
+    def _get_data_from_item(self):
+        values = {}
+        if not self._item:
+            return values
+        for name, field in self._config.get_fields().iteritems():
+            try:
+                values[name] = getattr(self._item, name)
+            except AttributeError:
+                values[name] = None
+        return values
+
+
+    def serialize(self, data):
+        """Returns a dictionary with serialized data from the forms
+        item.  The return dictionary is suitable for htmlfill to prefill
+        the rendered form. The dictionary will include all attributes
+        and relations values of the items. The key in the dictionary is
+        the name of the relation/attribute. In case of relations the
+        value in the dictionary is the "id" value of the related item.
+        If no item is present then return a empty dict.
+
+        :returns: Dictionary with serialized values of the item.
+
+        """
+        serialized = {}
+        if not data:
+            return serialized
+        for name, field in self._config.get_fields().iteritems():
+            try:
+                value = data.get(name)
+                if value is None:
+                    serialized[name] = ""
+                elif isinstance(value, list):
+                    vl = []
+                    for v in value:
+                        try:
+                            vl.append(v.id)
+                        except AttributeError:
+                            vl.append(v)
+                    serialized[name] = vl
+                else:
+                    try:
+                        serialized[name] = value.id
+                    except AttributeError:
+                        serialized[name] = value
+            except AttributeError:
+                log.warning('Can not get value for field "%s". '
+                            'The field is no attribute of the item' % name)
+                serialized[name] = ""
+        return serialized
 
     def _build_fields(self):
         """Returns a dictionary with all Field instanced which are
@@ -122,8 +185,7 @@ class Form(object):
         """
         fields = {}
         for name, field in self._config.get_fields().iteritems():
-            fa_field = self.fs[name]
-            fields[name] = Field(self, field, fa_field, self._translate)
+            fields[name] = Field(self, field, self._translate)
         return fields
 
     def has_errors(self):
@@ -161,7 +223,7 @@ class Form(object):
         self.current_page = page
         renderer = FormRenderer(self, self._translate)
         form = renderer.render(values)
-        return htmlfill.render(form, values or self.data)
+        return htmlfill.render(form, values or self.serialize(self.data))
 
     def _add_error(self, fieldname, error):
         field = self.get_field(fieldname)
@@ -177,25 +239,64 @@ class Form(object):
         :field: configuration of the field
         :value: value to be converted
         """
-        # Handle missing value. Currently we just return None in case
-        # that the provided value is an empty String
-        if value == "":
-            return None
+        relation_names = {}
+        try:
+            mapper = sa.orm.object_mapper(self._item)
+            relation_properties = filter(
+                lambda p: isinstance(p, sa.orm.properties.RelationshipProperty),
+                mapper.iterate_properties)
+            for prop in relation_properties:
+                relation_names[prop.key] = prop
+        except sa.orm.exc.UnmappedInstanceError:
+            if not self._item:
+                pass # The form is not mapped to an item.
+            else:
+                raise
 
-        dtype = field.type
-        if dtype == 'integer':
+        converted = ""
+        dtype = field.get_type()
+        if dtype == 'string':
             try:
-                return int(value)
+                converted = value
+            except ValueError:
+                msg = "%s is not a string value." % value
+                self._add_error(field.name, msg)
+        elif dtype == 'integer':
+            if not value:
+                return None
+            try:
+                converted = int(value)
             except ValueError:
                 msg = "%s is not a integer value." % value
                 self._add_error(field.name, msg)
         elif dtype == 'float':
+            if not value:
+                return None
             try:
-                return float(value)
+                converted = float(value)
             except ValueError:
                 msg = "%s is not a float value." % value
                 self._add_error(field.name, msg)
+        elif dtype == 'boolean':
+            if not value:
+                return None
+            try:
+                converted = value in ['True', '1', 't']
+            except ValueError:
+                msg = "%s is not a boolean value." % value
+                self._add_error(field.name, msg)
+        elif dtype == 'file':
+            try:
+                # filename = value.filename
+                converted = value.file.read()
+            except AttributeError:
+                return None
+            except ValueError:
+                msg = "%s is not a file value." % value
+                self._add_error(field.name, msg)
         elif dtype == 'date':
+            if not value:
+                return None
             try:
                 #@TODO: Support other dateformats that ISO8601
                 y, m, d = value.split('-')
@@ -203,14 +304,42 @@ class Form(object):
                 m = int(m)
                 d = int(d)
                 try:
-                    return datetime.date(y, m, d)
+                    converted = datetime.date(y, m, d)
                 except ValueError, e:
                     msg = "%s is an invalid date (%s)" % (value, e)
                     self._add_error(field.name, msg)
             except:
                 msg = "%s is not a valid date format." % value
                 self._add_error(field.name, msg)
-        return value
+        # Reltation handling
+        elif dtype == 'manytoone':
+            try:
+                db = self._dbsession
+                rel = relation_names[field.name].mapper.class_
+                if value in ("", None):
+                    converted = None
+                else:
+                    value = db.query(rel).filter(rel.id == int(value)).one()
+                    converted = value
+            except ValueError:
+                msg = "Reference value '%s' must be of type integer" % value
+                self._add_error(field.name, msg)
+        elif dtype in ['onetomany', 'manytomany']:
+            if not value:
+                return []
+            try:
+                values = []
+                db = self._dbsession
+                rel = relation_names[field.name].mapper.class_
+                for v in value:
+                    values.append(db.query(rel).filter(rel.id == int(v)).one())
+                converted = values
+            except ValueError:
+                msg = "Reference value '%s' must be of type integer" % value
+                self._add_error(field.name, msg)
+        log.debug("Converted value '%s' (%s) of field '%s' (%s)"
+                  % (converted, type(converted), field.name, dtype))
+        return converted
 
     def validate(self, submitted):
         """Returns True if the validation succeeds else False.
@@ -235,32 +364,20 @@ class Form(object):
 
         """
 
+        submitted = variabledecode.variable_decode(submitted)
         # This dictionary will contain the converted data
         values = {}
-        fa_validated = False
         # 1. Iterate over all fields and start the validation.
         log.debug('Submitted values: %s' % submitted)
-        for fieldname in submitted.keys():
-            try:
-                field = self._config.get_field(fieldname)
-            except KeyError:
-                # @TODO:
-                # For 1:1 relations FA modifies the fieldname on
-                # rendering to the name of the Foreign Key. This leads
-                # to problems on validation when the validation code
-                # tries to access a field based on the submitted data
-                # and raises a KeyError as there is no field with the
-                # name of the FK.
-                # In this case we we just add the subbmitted data to the
-                # validated data, to make the prefilling work after rendering.
-                values[fieldname] = submitted.get(fieldname)
-                log.warning('Found field "%s" in submitted data,'
-                            ' while validating data for "%s" which is'
-                            ' not a configured field'
-                            % (fieldname, repr(self._item)))
-                continue
+        for fieldname, field in self.fields.iteritems():
+            field = self.fields[fieldname]
+            rules = field.get_rules()
+            # TODO: clean the submitted data. Only use values for fields
+            # which are actually configured in the field (torsten)
+            # <2013-07-30 09:02>
+
             # 3. Prevalidation
-            for rule in field.rules:
+            for rule in rules:
                 if rule.mode != 'pre':
                     continue
                 result = rule.evaluate(submitted)
@@ -268,32 +385,12 @@ class Form(object):
                     self._add_error(fieldname, rule.msg)
 
             # 4. Basic type conversations, Defaults to String
-            # Validation can happen in two variations:
-            #
-            # 4.1 If item is None (No sqlalchemy mapped item is provided),
-            # then convert each field in the Form into its python type.
-            #
-            # 4.2 If an item was provided, than use the FormAlchemy
-            # validation once for the whole fieldset. After validation
-            # was done save the data into the internal data dictionary.
-            if self._item is None:
-                values[fieldname] = self._convert(field, submitted[fieldname])
-            else:
-                if not fa_validated:
-                    self.fs.rebind(self._item, data=submitted)
-                    fa_valid = self.fs.validate()
-                    fa_validated = True
-                    if not fa_valid:
-                        # Collect all errors form formalchemy
-                        for err_field, err_msg in self.fs.errors.iteritems():
-                            self._add_error(err_field.key, err_msg)
-                if not fa_valid:
-                    values[fieldname] = self.fs[fieldname].raw_value
-                else:
-                    values[fieldname] = self.fs[fieldname].value
+            if not field.is_readonly():
+                # Only add the value if the field is not marked as readonly
+                values[fieldname] = self._convert(field, submitted.get(fieldname))
 
             # 5. Postvalidation
-            for rule in field.rules:
+            for rule in rules:
                 if rule.mode != 'post':
                     continue
                 result = rule.evaluate(values)
@@ -328,54 +425,17 @@ class Form(object):
         if self.has_errors():
             raise StateError('Saving is not possible if form has errors')
 
-        # @FIXME: _item is only set when this form is used in connection
-        # with an SQLAlchemy mapped item.If the form is used as normal
-        # form _item is none. This is not consistent. There seems to be
-        # more options to fix this:
-        # 1. Raise exception when calling save if there is no _item.
-        # 2. Make sure there is always an item
-        # 3. Save the item if there is an item, else ignore. Return None
-        # in both cases.
+        # Only save if there is actually an item.
         if self._item is not None:
-            try:
-                self._save()
-            except:
-                self.fs.sync()
-            return self._item
-
-    def _save(self):
-        mapper = sa.orm.object_mapper(self._item)
-        relation_properties = filter(
-            lambda p: isinstance(p, sa.orm.properties.RelationshipProperty),
-            mapper.iterate_properties)
-
-        relation_names = {}
-        for prop in relation_properties:
-            relation_names[prop.key] = prop
-
-        # related_classes = [prop.mapper.class_ for prop in relation_properties]
-        # related_tables = [prop.target for prop in relation_properties]
-
-        for key, value in self.data.iteritems():
-            relation = relation_names.get(key)
-            if relation:
-                log.info('Todo: Implement setting relation')
-                li = self._load_relations(relation.mapper.class_, value)
-                try:
-                    setattr(self._item, key, li)
-                except:
-                    log.exception('Error while setting %s with %s' % (key, li))
-            else:
+            # TODO: Iterate over fields here. Fields should know their value
+            # and if they are a relation or not (torsten) <2013-07-24 23:24>
+            for key, value in self.data.iteritems():
                 setattr(self._item, key, value)
-
-        self._dbsession.add(self._item)
-
-    def _load_relations(self, relation, values):
-        loaded = []
-        for value in values:
-            r = self._dbsession.query(relation).filter(relation.id == value).one()
-            loaded.append(r)
-        return loaded
+            # If the item has no id, then we assume it is a new item. So
+            # add it to the database session.
+            if not self._item.id:
+                self._dbsession.add(self._item)
+        return self._item
 
 
 class Field(object):
@@ -383,7 +443,7 @@ class Field(object):
     provide a common interface for the renderer independent to the
     underlying implementation detail of the field."""
 
-    def __init__(self, form, config, fa_field, translate):
+    def __init__(self, form, config, translate):
         """Initialize the field with the given field configuration.
 
         :config: Field configuration
@@ -391,7 +451,7 @@ class Field(object):
         """
         self._form = form
         self._config = config
-        self._fa_field = fa_field
+        self.sa_property = self._get_sa_property()
         self._translate = translate
         self.renderer = get_renderer(self, translate)
         self._errors = []
@@ -400,11 +460,88 @@ class Field(object):
         """Make attributes from the configuration directly available"""
         return getattr(self._config, name)
 
-    def get_value(self):
-        return self._fa_field.raw_value
+    def _get_sa_mapped_class(self):
+        # TODO: Raise Exception if this field is not a relation. (None)
+        # <2013-07-25 07:44>
+        return self.sa_property.mapper.class_
+
+    def _get_sa_property(self):
+        if not self._form._item:
+            return None
+        mapper = sa.orm.object_mapper(self._form._item)
+        for prop in mapper.iterate_properties:
+            if prop.key == self.name:
+                return prop
+
+    def get_type(self):
+        """Returns the datatype of the field."""
+        if self._config.type:
+            return self._config.type
+        if self.sa_property:
+            try:
+                column = self.sa_property.columns[0]
+                dtype = str(column.type)
+                if dtype in ["VARCHAR", "TEXT"]:
+                    return "string"
+                elif dtype == "DATE":
+                    return "date"
+                elif dtype == "INTEGER":
+                    return "integer"
+                elif dtype == "BOOLEAN":
+                    return "boolean"
+                else:
+                    log.warning('Unhandled datatype: %s' % dtype)
+            except AttributeError:
+                return self.sa_property.direction.name.lower()
+        return "string"
+
+    def get_rules(self):
+        """Returns a list of configured rules for the field"""
+        rules = self.rules
+        if self.is_required():
+            parser = Parser()
+            expr = "bool($%s)" % self.name
+            msg = "This field is required. You need to provide a value"
+            mode = "pre"
+            expr = parser.parse(expr)
+            rules.append(Rule(expr, msg, mode))
+        return rules
+
+    def get_value(self, default=None, expand=False):
+        value = self._form.data.get(self._config.name, default)
+        if expand:
+            if not isinstance(value, list):
+                value = [value]
+            ex_values = []
+            options = self.get_options()
+            for opt in options:
+                for v in value:
+                    if str(v) == str(opt[1]):
+                        ex_values.append("%s" % opt[0])
+            return ", ".join(ex_values)
+        else:
+            return value
 
     def get_options(self):
-        return self._config.options
+        user_defined_options = self._config.options
+        print user_defined_options
+        if user_defined_options:
+            return user_defined_options
+        elif self._form._dbsession:
+            # Get mapped clazz for the field
+            options = []
+            if self.get_type() == 'manytoone':
+                options.append(("None", ""))
+            clazz = self._get_sa_mapped_class()
+            items = self._form._dbsession.query(clazz)
+            options.extend([(item, item.id) for item in items])
+            return options
+        else:
+            # TODO: Try to get the session from the item. Ther must be
+            # somewhere the already bound session. (torsten) <2013-07-23 00:27>
+            log.warning('No db connection configured for this form. Can '
+                        'not load options')
+            return []
 
     def add_error(self, error):
         self._errors.append(error)
@@ -416,12 +553,21 @@ class Field(object):
         """Returns the rendererd HTML for the field"""
         return self.renderer.render()
 
+    def is_relation(self):
+        return isinstance(self.sa_property,
+                          sa.orm.RelationshipProperty)
+
     def is_required(self):
         """Returns true if either the required flag of the field
-        configuration is set or the formalchemy field is required."""
-        return self.required or self._fa_field.is_required()
+        configuration is set or the field is required in the underlying
+        datamodel"""
+        req = False
+        if isinstance(self.sa_property, sa.orm.RelationshipProperty) is False \
+           and self.sa_property:
+            req = (self.sa_property.columns[0].nullable is False)
+        return self.required or req
 
     def is_readonly(self):
         """Returns true if either the readonly flag of the field
-        configuration is set or the formalchemy field is readonly."""
-        return self.readonly or self._fa_field.is_readonly()
+        configuration is set or the whole form is marked as readonly"""
+        return self.readonly or False
